/*
 * This file is part of Bluhell Firewall Browser Extension,
 * Copyright (c) 2016, Diego Casorran. All rights reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

(function(scope) {
'use strict';

let {classes:Cc,interfaces:Ci,utils:Cu} = Components, addon;

Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

function rsc(n) 'resource://' + addon.tag + '/' + n;
function LOG(m) (m = addon.name + ' Message @ '
	+ (new Date()).toISOString() + "\n> " + m,
		dump(m + "\n"), Services.console.logStringMessage(m));

let ia = Services.appinfo.ID[3],
	wt = 5==ia?'mail:3pane'
		:'navigator:browser';

XPCOMUtils.defineLazyGetter(scope, 'aUI', function() {
	if (3!=ia) try {
		var { CustomizableUI } = Cu.import('resource:///modules/CustomizableUI.jsm', {});
	} catch(e) {
		Cu.reportError(e);
	}
	return typeof CustomizableUI !== 'undefined' && CustomizableUI;
});

let i$ = {
	enabled: true,
	
	toggle: function() {
		
		if((this.enabled = !this.enabled)) {
			Services.ppmm.broadcastAsyncMessage('bluhell:' + addon.fsid, 'enable');
		}
		else {
			Services.ppmm.broadcastAsyncMessage('bluhell:' + addon.fsid, 'disable');
		}
		
		if(3!=ia) this.wmf(w=>addon.wms.get(w).onCustomize());
		
		return this.enabled;
	},
	wmf: function(callback) {
		let windows = Services.wm.getEnumerator(wt);
		while(windows.hasMoreElements())
			callback(windows.getNext()
				.QueryInterface(Ci.nsIDOMWindow));
	},
	onOpenWindow: function(aWindow) {
		loadIntoWindowStub(aWindow
			.QueryInterface(Ci.nsIInterfaceRequestor)
			.getInterface(Ci.nsIDOMWindow));
	},
	onCloseWindow: function() {},
	onWindowTitleChange: function() {}
};

function loadIntoWindow(window) {
	if(wt!=window.document.documentElement
		.getAttribute("windowtype"))
			return;
	
	if(3==ia) {
		let NativeWindow = window.NativeWindow;
		let mID = NativeWindow.menu.add({
			name: i$.classDescription,
			icon: addon.resourceURI.spec + 'icon16.png',
			checkable: true,
			callback: function() {
				NativeWindow.menu.update(mID, { checked: i$.toggle() });
			}
		});
		NativeWindow.menu.update(mID, { checked: !0 });
		addon.wms.set(window,mID);
		return;
	}
	
	function $(n) window.document.getElementById(n);
	let e = (n,a) => {
		if(!(n = window.document.createElement(n)))
			return null;
		
		for(let x in a)n.setAttribute(x,''+a[x]);
		return n;
	};
	let setIcon = (e) => {
		let s = tn.getAttribute('cui-areatype')=='menu-panel'? 32:16;
		tn.setAttribute('image', rsc('icon'+s+(e?'':'n')+'.png'));
		tn.setAttribute('tooltiptext',addon.name+' '+addon.version+' [O'+(e?'N':'FF')+']');
	};
	
	let wmsData = {
		TBBHandler: () => setIcon(i$.toggle()),
		onCustomize: () => setIcon(i$.enabled)
	};
	addon.wms.set(window,wmsData);
	
	let m = addon.tag+'-toolbar-button',
	tn = e('toolbarbutton',{
			id:m,label:addon.name,class:'toolbarbutton-1',
			tooltiptext:addon.name+' '+addon.version,
			image:rsc('icon16.png')
		});
	tn.addEventListener('command', wmsData.TBBHandler, !1);
	let gNavToolbox = !aUI && (window.gNavToolbox || $("mail-toolbox"));
	if(gNavToolbox && gNavToolbox.palette) {
		gNavToolbox.palette.appendChild(tn);
		
		if(!addon.branch.getPrefType("version")) {
			let nv = $('nav-bar') || $('mail-bar3');
			if( nv ) {
				nv.insertItem(m, null, null, false);
				nv.setAttribute("currentset", nv.currentSet);
				window.document.persist(nv.id, "currentset");
			}
		} else {
			for(let tb of window.document.querySelectorAll("toolbar[currentset]")) {
				let cs = tb.getAttribute("currentset").split(","),
					bp = cs.indexOf(m) + 1;
				
				if(bp) {
					let at = null, f = [],
					xul={spacer:1,spring:1,separator:1};
					cs.splice(bp).some(function(id)
						(at=$(id))?!0:(f.push(id),!1));
					at&&f.length&&f.forEach(function(n)xul[n]
						&&(at=at&&at.previousElementSibling));
					tb.insertItem(m, at, null, false);
					return true;
				}
			}
		}
	} else {
		window.addEventListener('aftercustomization', wmsData.onCustomize );
	}
	gNavToolbox = e = undefined;
	return tn;
}

function loadIntoWindowStub(domWindow) {
	
	if(domWindow.document.readyState == "complete") {
		loadIntoWindow(domWindow);
	} else {
		domWindow.addEventListener(3==ia? 'UIReady':"load",
			function _(ev) (this.removeEventListener(ev.type,_,!1),
				loadIntoWindow(this)), !1);
	}
}

function unloadFromWindow(window) {
	if(3==ia) {
		let mID = addon.wms.get(window);
		window.NativeWindow.menu.remove(mID);
		addon.wms.delete(window);
		return;
	}
	
	let $ = function(n) window.document.getElementById(n);
	let btnId = addon.tag+'-toolbar-button',btn= $(btnId);
	
	if(addon.wms.has(window)) {
		let wmsData = addon.wms.get(window);
		
		if(btn) {
			btn.removeEventListener('command',wmsData.TBBHandler,false);
		}
		window.removeEventListener('aftercustomization', wmsData.onCustomize );
		addon.wms.delete(window);
	}
	
	if(btn) {
		btn.parentNode.removeChild(btn);
	} else {
		let gNavToolbox = window.gNavToolbox || $('mail-toolbox');
		if(gNavToolbox && gNavToolbox.palette) {
			for each(let node in gNavToolbox.palette) {
				if(node && node.id == btnId) {
					gNavToolbox.palette.removeChild(node);
					break;
				}
			}
		}
	}
}

function startup(aData) {
	let tmp = {};
	Cu.import("resource://gre/modules/AddonManager.jsm", tmp);
	tmp.AddonManager.getAddonByID(aData.id,data => {
		let io = Services.io;
		
		addon = {
			id: data.id,
			name: data.name,
			version: data.version,
			tag: data.name.toLowerCase().replace(/[^\w]/g,''),
			resourceURI: aData.resourceURI,
			wms: new WeakMap()
		};
		addon.branch = Services.prefs.getBranch('extensions.'+addon.tag+'.');
		
		io.getProtocolHandler("resource")
			.QueryInterface(Ci.nsIResProtocolHandler)
			.setSubstitution(addon.tag,
				io.newURI(__SCRIPT_URI_SPEC__+'/../',null,null));
		
		if(aUI) {
			aUI.createWidget({
				type: 'custom',
				defaultArea: aUI.AREA_NAVBAR,
				id:addon.tag+'-toolbar-button',
				onBuild:d=>loadIntoWindow(d.defaultView)
			});
		} else {
			i$.wmf(loadIntoWindowStub);
			Services.wm.addListener(i$);
		}
		
		addon.fsid = addon.id.replace(/[^\w-]/g, '') + '-' + (Math.random()*Date.now()).toString(36);
		Services.ppmm.loadProcessScript(rsc('content.js?id=' + addon.fsid), true);
		
		addon.branch.setCharPref('version', addon.version);
	});
}

function shutdown(data, reason) {
	if(reason == APP_SHUTDOWN)
		return;
	
	Services.ppmm.broadcastAsyncMessage('bluhell:' + addon.fsid, 'shutdown');
	Services.ppmm.removeDelayedProcessScript(rsc('content.js?id=' + addon.fsid));
	
	if(aUI) {
		aUI.destroyWidget(addon.tag+'-toolbar-button');
	} else {
		Services.wm.removeListener(i$);
	}
	i$.wmf(unloadFromWindow);
	
	Services.io.getProtocolHandler("resource")
		.QueryInterface(Ci.nsIResProtocolHandler)
		.setSubstitution(addon.tag,null);
	
	addon = undefined;
}

let wrap = function(f) {
	f.prototype = undefined;
	return {value:Object.freeze(f.bind(null))};
};
Object.defineProperties(scope, {
	startup   : wrap(startup),
	shutdown  : wrap(shutdown),
	install   : wrap(function() {}),
	uninstall : wrap(function() {})
});
})(this);
